@@ -11,7 +11,7 @@ use Data::Dumper ();
 
 $ENV{ANSI_COLORS_DISABLED} = 1 if $^O eq 'MSWin32';
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 my $org_execute               = \&DBI::st::execute;
 my $org_bind_param            = \&DBI::st::bind_param;
@@ -23,6 +23,8 @@ my $org_db_selectrow_array    = \&DBI::db::selectrow_array;
 my $has_mysql = eval { require DBD::mysql; 1 } ? 1 : 0;
 my $pp_mode   = $INC{'DBI/PurePerl.pm'} ? 1 : 0;
 
+my $supports_explain = $has_mysql && eval { require Text::ASCIITable; 1 } ? 1 : 0;
+
 our %SKIP_PKG_MAP = (
     'DBIx::QueryLog' => 1,
 );
@@ -75,7 +77,7 @@ sub unimport {
 *disable = *end   = \&unimport;
 
 my $container = {};
-for my $accessor (qw/logger threshold probability skip_bind color useqq compact/) {
+for my $accessor (qw/logger threshold probability skip_bind color useqq compact explain/) {
     no strict 'refs';
     *{__PACKAGE__."::$accessor"} = sub {
         use strict 'refs';
@@ -109,6 +111,12 @@ sub _st_execute {
         }
         $sth->{private_DBIx_QueryLog_params} = undef;
 
+        my $explain;
+        if ($supports_explain and $container->{explain} || $ENV{DBIX_QUERYLOG_EXPLAIN}) {
+            my $dbh = $sth->{Database};
+            $explain = _explain($dbh, $ret, \@params, \@types);
+        }
+
         unless (($container->{skip_bind} || $ENV{DBIX_QUERYLOG_SKIP_BIND}) && @params) {
             my $dbh = $sth->{Database};
             $ret = _bind($dbh, $ret, \@params, \@types);
@@ -118,7 +126,7 @@ sub _st_execute {
         my $res = $wantarray ? [$org->($sth, @_)] : scalar $org->($sth, @_);
         my $time = sprintf '%.6f', tv_interval $begin, [gettimeofday];
 
-        $class->_logging($ret, $time, \@params);
+        $class->_logging($ret, $time, \@params, $explain);
 
         return $wantarray ? @$res : $res;
     };
@@ -154,6 +162,12 @@ sub _select_array {
         }
 
         my $ret = ref $stmt ? $stmt->{Statement} : $stmt;
+
+        my $explain;
+        if ($supports_explain and $container->{explain} || $ENV{DBIX_QUERYLOG_EXPLAIN}) {
+            $explain = _explain($dbh, $ret, \@bind);
+        }
+
         unless (($container->{skip_bind} || $ENV{DBIX_QUERYLOG_SKIP_BIND}) && @bind) {
             $ret = _bind($dbh, $ret, \@bind);
         }
@@ -168,7 +182,7 @@ sub _select_array {
         }
         my $time = sprintf '%.6f', tv_interval $begin, [gettimeofday];
 
-        $class->_logging($ret, $time, \@bind);
+        $class->_logging($ret, $time, \@bind, $explain);
 
         if ($is_selectrow_array) {
             return $wantarray ? @$res : $res;
@@ -194,6 +208,12 @@ sub _db_do {
         }
 
         my $ret = $stmt;
+
+        my $explain;
+        if ($supports_explain and $container->{explain} || $ENV{DBIX_QUERYLOG_EXPLAIN}) {
+            $explain = _explain($dbh, $ret, \@bind);
+        }
+
         unless (($container->{skip_bind} || $ENV{DBIX_QUERYLOG_SKIP_BIND}) && @bind) {
             $ret = _bind($dbh, $ret, \@bind);
         }
@@ -202,12 +222,46 @@ sub _db_do {
         my $res = $wantarray ? [$org->($dbh, $stmt, $attr, @bind)] : scalar $org->($dbh, $stmt, $attr, @bind);
         my $time = sprintf '%.6f', tv_interval $begin, [gettimeofday];
 
-        $class->_logging($ret, $time, \@bind);
+        $class->_logging($ret, $time, \@bind, $explain);
 
         return $wantarray ? @$res : $res;
     };
 }
 
+sub _explain {
+    my ($dbh, $ret, $params, $types) = @_;
+    $types ||= [];
+
+    if ($dbh->{Driver}{Name} ne 'mysql') {
+        return undef;
+    }
+
+    return undef unless $ret =~ m|^
+        \s*                # white space
+        (?: /\* .*? \*/ )* # /* ... */
+        \s*                # while space
+        SELECT
+    |ixms;
+
+    return sub {
+        my %args = @_;
+        no warnings qw(redefine prototype);
+        local *DBI::st::execute = $org_execute; # suppress duplicate logging
+
+        my $sql = 'EXPLAIN ' . _bind($dbh, $ret, $params, $types);
+        my $sth = $dbh->prepare($sql);
+        $sth->execute;
+
+        return $sth->fetchall_arrayref(+{}) unless defined $args{print} and $args{print};
+
+        my $t = Text::ASCIITable->new();
+        $t->setCols(@{$sth->{NAME}});
+        $t->addRow(map { defined($_) ? $_ : 'NULL' } @$_) for @{$sth->fetchall_arrayref};
+
+        return $t;
+    };
+}
+
 sub _bind {
     my ($dbh, $ret, $params, $types) = @_;
     $types ||= [];
@@ -236,7 +290,7 @@ sub _bind {
 }
 
 sub _logging {
-    my ($class, $ret, $time, $bind_params) = @_;
+    my ($class, $ret, $time, $bind_params, $explain) = @_;
 
     my $threshold = $container->{threshold} || $ENV{DBIX_QUERYLOG_THRESHOLD};
     return unless !$threshold || $time > $threshold;
@@ -319,6 +373,7 @@ sub _logging {
         $caller->{file}, $caller->{line};
 
     if (my $logger = $container->{logger}) {
+        my %explain = $explain ? (explain => $explain->()) : ();
         $logger->log(
             level   => $LOG_LEVEL,
             message => $message,
@@ -327,12 +382,14 @@ sub _logging {
                 time        => $time,
                 sql         => $sql,
                 bind_params => $bind_params,
+                %explain,
                 %$caller,
             },
         );
     }
     else {
         if (ref $OUTPUT eq 'CODE') {
+            my %explain = $explain ? (explain => $explain->()) : ();
             $OUTPUT->(
                 level       => $LOG_LEVEL,
                 message     => $message,
@@ -340,11 +397,12 @@ sub _logging {
                 time        => $time,
                 sql         => $sql,
                 bind_params => $bind_params,
+                %explain,
                 %$caller,
             );
         }
         else {
-            print {$OUTPUT} $message;
+            print {$OUTPUT} $message, $explain ? $explain->(print => 1) : ();
         }
     }
 }
@@ -436,6 +494,23 @@ Compaction SQL.
 
 And, you can also specify C<< DBIX_QUERYLOG_COMPACT >> environment variable.
 
+=item explain
+
+B<< EXPERIMENTAL >>
+
+Logged Explain. (only supports mysql now)
+
+  DBIx::QueryLog->explain(1);
+  my $row = $dbh->do(...);
+  # => SELECT * FROM peaple WHERE user_id = '1986'
+  #  .----------------------------------------------------------------------------------------------.
+  #  | id | select_type | table  | type  | possible_keys | key     | key_len | ref   | rows | Extra |
+  #  +----+-------------+--------+-------+---------------+---------+---------+-------+------+-------+
+  #  |  1 | SIMPLE      | peaple | const | PRIMARY       | PRIMARY |       4 | const |    1 |       |
+  #  '----+-------------+--------+-------+---------------+---------+---------+-------+------+-------'
+
+And, you can also specify C<< DBIX_QUERYLOG_EXPLAIN >> environment variable.
+
 =back
 
 =head1 TIPS
